public class Block extends PhysicsElement  {
   private static int id=0;  // Block identification
   private final float mass;
   private float width;		// original code had this warning
   private Vector2D pos_t;
   private Vector2D pos_tPlusDelta = new Vector2D();
   private Vector2D speed_t;
   private Vector2D speed_tPlusDelta = new Vector2D();
   private Spring leftSpring, rightSpring;
   private Vector2D force = new Vector2D();
   private Vector2D a = new Vector2D();
 
   private Block(){   // nobody can create a block without state
     super(id++);
     mass=0;
   }
   public Block(float mass, float width, Vector2D position, Vector2D speed){
      super(id++);
      this.mass = mass;
      this.width = width;
      pos_t = position;
      this.speed_t = speed;
      leftSpring = rightSpring = null;
   }
   public Block(float mass, float width, Vector2D position, Vector2D speed, Spring lSpring, Spring rSpring){
     this(mass, width, position, speed);
     leftSpring = lSpring;
     rightSpring = rSpring;
   }
   public void attachLeftSpring (Spring lSpring) {
     leftSpring = lSpring;
   }
   public void attachRightSpring (Spring rSpring) {
     rightSpring = rSpring;
   }
   public Vector2D getPosition() {
      return pos_t;
   }
   
   ///////////////////////////////////////
   public String getForce() {
	      return "["+force.getx()+","+force.gety()+"]";
   }
   public String getAccel() {
	      return "["+a.getx()+","+a.gety()+"]";
   }
   ////////////////////////////////////////
   
   public void computeNextState(double delta_t) {
	 force.setCoords(0, 0);
	 //get total force applied to the block
	 if (leftSpring != null) this.force = leftSpring.getForce(this);
     if (rightSpring != null) this.force.sumaVector(rightSpring.getForce(this));   
     a.setCoords(force.getx()/mass, force.gety()/mass);
     
     pos_tPlusDelta.setCoords(0.0, 0.0);		//calculate next pos from 0
     pos_tPlusDelta.sumaVector(pos_t);
     pos_tPlusDelta.sumaVector(speed_t.speedtoPos(delta_t));
     pos_tPlusDelta.sumaVector(a.acceltoPos(delta_t));
     speed_tPlusDelta.setCoords(0, 0);			//calculate next speed from 0
     speed_tPlusDelta.sumaVector(speed_t);
     speed_tPlusDelta.sumaVector(a.acceltoSpeed(delta_t));
   }
   public void updateState(){
	 pos_t.setCoords(0, 0);
	 pos_t.sumaVector(pos_tPlusDelta);
     speed_t.setCoords(0, 0);
     speed_t.sumaVector(speed_tPlusDelta);
   }
   public String getDescription() {
      return "Block_" + getId()+": x,y";
   }
   public String getState() {
      return getPosition().getx() + "," + getPosition().gety();
   }
}
