import java.io.PrintStream;

public class Spring extends PhysicsElement  {
   private static int id=0;  // Spring identification
   private float restLength;
   private final float stiffness;
   private Block leftBlock, rightBlock; //clockwise turn, if neccesary
   
   PrintStream Out = new PrintStream(System.out);

   private Spring(){  // nobody can create a block without state
     super(id++);
     stiffness=0;
   }
   public Spring(float restLength, float stiffness){
     super(id++);
     this.restLength = restLength;
     this.stiffness = stiffness;
     leftBlock = rightBlock = null;
   }
   public Spring(float restLength, float stiffness, Block lBlock, Block rBlock){
     this(restLength, stiffness);
     leftBlock = lBlock;
     rightBlock = rBlock;
   }
   public void attachLeftBlock (Block lBlock) {
     leftBlock = lBlock;
   }
   public void attachRightBlock (Block rBlock) {
     rightBlock = rBlock;
   }
   public double getLength() {
     if (rightBlock == null || leftBlock == null) return restLength;
     Vector2D aux = new Vector2D();
     aux = rightBlock.getPosition().diffVector(leftBlock.getPosition());
     return aux.mag();
   }
   public Vector2D getForce(Block block) {
     Vector2D forceAux = new Vector2D();
     if (block == leftBlock)
    	 forceAux = leftBlock.getPosition().diffVector(rightBlock.getPosition());
     else if (block == rightBlock)
    	 forceAux = rightBlock.getPosition().diffVector(leftBlock.getPosition());
     else forceAux.setCoords(0.0,0.0);
     
     double forceMag = (getLength()-restLength)*stiffness;
     double ang = forceAux.getAngle();			//in radians, always in the 1st o 2nd quadrant
     int quad = forceAux.getQuadrant();			//where the vector points
     switch (quad){								//considering the "-" of -k+x and the sign of ang
     	case 1:	forceAux.setCoords(forceMag*Math.cos(ang)*-1, forceMag*Math.sin(ang)*-1); break;
     	case 2:	forceAux.setCoords(forceMag*Math.cos(ang)*-1, forceMag*Math.sin(ang)*-1); break;
     	case 3:	forceAux.setCoords(forceMag*Math.cos(ang), forceMag*Math.sin(ang)); break;
     	case 4:	forceAux.setCoords(forceMag*Math.cos(ang), forceMag*Math.sin(ang)); break;
     	default:forceAux.setCoords(0,0); break;
     }
     return forceAux;
   }
   public void computeNextState(double delta_t) {
   }
   public void updateState(){
   }
   public String getDescription() {
      return "Spring_"+ getId()+": x0,y0,x1,y1";
   }
   public String getState() {
      String s;
      if (leftBlock != null) 
         s = leftBlock.getState()+",";
      else if (rightBlock != null)
      		  s = "pending";			//the spring has one loose end
      else  
              s = "0,";  // up to here left end is determined 
      if (rightBlock != null)
         s += rightBlock.getState();
      else if (leftBlock != null)
  		  	  s = "pending";			//the spring has one loose end
      else
           s += "0"; 
      return s;
   }
}
          
    
    
           
      